use crate::{schema,UIDCError};
use microrm::prelude::*;
use ring::signature::{Ed25519KeyPair, KeyPair};
use sha2::Digest;
use std::collections::HashMap;

#[derive(Debug)]
pub enum KeyError {
    Plain(&'static str),
}

pub struct KeyStore<'a> {
    db: &'a microrm::DB,
    qi: microrm::QueryInterface<'a>,
    keys: HashMap<String, Ed25519KeyPair>,
}

impl<'a> KeyStore<'a> {
    pub fn new(db: &'a microrm::DB) -> Self {
        Self {
            db,
            qi: db.query_interface(),
            keys: HashMap::new(),
        }
    }

    pub fn generate_in(&self, realm_id: schema::RealmID) -> Result<String, UIDCError> {
        let mut rng = ring::rand::SystemRandom::new();
        let sign_generated = Ed25519KeyPair::generate_pkcs8(&mut rng);

        if let Err(_) = sign_generated {
            return Err(KeyError::Plain("Failed to generate key").into())
        }
        let sign_generated = sign_generated.unwrap();

        let keydata = sign_generated.as_ref().to_owned();

        let loaded_key = Ed25519KeyPair::from_pkcs8(keydata.as_slice()).unwrap();
        let pubkey = loaded_key.public_key();

        let mut key_hasher = sha2::Sha256::new();
        key_hasher.update(&pubkey.as_ref());
        let mut key_id = base64::encode(key_hasher.finalize());
        key_id.truncate(16);

        self.qi
            .add(&schema::Key {
                realm: realm_id,
                key_id: key_id.clone(),
                keydata,
            })?;

        Ok(key_id)
    }
}

pub fn inspect(db: &microrm::DB, realm_id: schema::RealmID) -> Result<(), UIDCError> {
    let qi = db.query_interface();
    let cs = KeyStore::new(db);
    println!("Keystore loaded.");

    println!("Retrieving keys realm...");
    let keys = qi
        .get()
        .by(schema::Key::Realm, &realm_id)
        .all()?;

    for key in keys {
        println!("- [{:20}]", key.key_id);
    }
    Ok(())
}

pub fn generate(db: &microrm::DB, realm_id: schema::RealmID) -> Result<(), UIDCError> {
    let qi = db.query_interface();
    let cs = KeyStore::new(db);
    match cs.generate_in(realm_id) {
        Ok(_) => (),
        Err(e) => {
            println!("Failed to generate key: {}", e);
        }
    }
    Ok(())
}
